import locale
import re
from mimetypes import common_types, types_map

from django.db import models

locales = \
    sorted(set(
        [
            code[:code.index('.')] if '.' in code else code
            for code in locale.locale_alias.values()
        ]
    ))
mimetypes = \
    sorted(set(
        list(types_map.values()) + list(common_types.values())
    ))


def _print_locale_enum():
    print('class LocaleCode(Enum):')
    for code in locales:
        print(f'    {code} = \'{code}\'')


def _print_mimetypes_enum():
    print('class MimeType(Enum):')
    for mimetype in mimetypes:
        key = re.sub('[\.\-\+/]', '_', mimetype)
        print(f'    {key} = \'{mimetype}\'')


class MediaType(models.TextChoices):
    image = 'image'
    video = 'video'
    audio = 'audio'


class OgpType(models.TextChoices):
    music_song = 'music.song'
    music_album = 'music.album'
    music_playlist = 'music.playlist'
    music_radio_station = 'music.radio_station'
    video_movie = 'video.movie'
    video_episode = 'video.episode'
    video_tv_show = 'video.tv_show'
    video_other = 'video.other'
    article = 'article'
    book = 'book'
    profile = 'profile'
    website = 'website'

    @classmethod
    def get_namespace(cls, value: str):
        prefix = {
            'music': 'https://ogp.me/ns/music#',
            'video': 'https://ogp.me/ns/video#',
            'article': 'https://ogp.me/ns/article#',
            'book': 'https://ogp.me/ns/book#',
            'profile': 'https://ogp.me/ns/profile#',
            'website': 'https://ogp.me/ns/website#',
        }
        key = value.split('.')[0]
        return f'{key}: {prefix[key]}'

    @property
    def namespace(self):
        return self.get_namespace(self.value)


class Locale(models.TextChoices):
    C = 'C'
    aa_DJ = 'aa_DJ'
    aa_ER = 'aa_ER'
    aa_ET = 'aa_ET'
    af_ZA = 'af_ZA'
    agr_PE = 'agr_PE'
    ak_GH = 'ak_GH'
    am_ET = 'am_ET'
    an_ES = 'an_ES'
    anp_IN = 'anp_IN'
    ar_AA = 'ar_AA'
    ar_AE = 'ar_AE'
    ar_BH = 'ar_BH'
    ar_DZ = 'ar_DZ'
    ar_EG = 'ar_EG'
    ar_IN = 'ar_IN'
    ar_IQ = 'ar_IQ'
    ar_JO = 'ar_JO'
    ar_KW = 'ar_KW'
    ar_LB = 'ar_LB'
    ar_LY = 'ar_LY'
    ar_MA = 'ar_MA'
    ar_OM = 'ar_OM'
    ar_QA = 'ar_QA'
    ar_SA = 'ar_SA'
    ar_SD = 'ar_SD'
    ar_SS = 'ar_SS'
    ar_SY = 'ar_SY'
    ar_TN = 'ar_TN'
    ar_YE = 'ar_YE'
    as_IN = 'as_IN'
    ast_ES = 'ast_ES'
    ayc_PE = 'ayc_PE'
    az_AZ = 'az_AZ'
    az_IR = 'az_IR'
    be_BY = 'be_BY'
    bem_ZM = 'bem_ZM'
    ber_DZ = 'ber_DZ'
    ber_MA = 'ber_MA'
    bg_BG = 'bg_BG'
    bhb_IN = 'bhb_IN'
    bho_IN = 'bho_IN'
    bho_NP = 'bho_NP'
    bi_VU = 'bi_VU'
    bn_BD = 'bn_BD'
    bn_IN = 'bn_IN'
    bo_CN = 'bo_CN'
    bo_IN = 'bo_IN'
    br_FR = 'br_FR'
    brx_IN = 'brx_IN'
    bs_BA = 'bs_BA'
    byn_ER = 'byn_ER'
    ca_AD = 'ca_AD'
    ca_ES = 'ca_ES'
    ca_FR = 'ca_FR'
    ca_IT = 'ca_IT'
    ce_RU = 'ce_RU'
    chr_US = 'chr_US'
    ckb_IQ = 'ckb_IQ'
    cmn_TW = 'cmn_TW'
    crh_UA = 'crh_UA'
    cs_CZ = 'cs_CZ'
    csb_PL = 'csb_PL'
    cv_RU = 'cv_RU'
    cy_GB = 'cy_GB'
    da_DK = 'da_DK'
    de_AT = 'de_AT'
    de_BE = 'de_BE'
    de_CH = 'de_CH'
    de_DE = 'de_DE'
    de_IT = 'de_IT'
    de_LI = 'de_LI'
    de_LU = 'de_LU'
    doi_IN = 'doi_IN'
    dv_MV = 'dv_MV'
    dz_BT = 'dz_BT'
    ee_EE = 'ee_EE'
    el_CY = 'el_CY'
    el_GR = 'el_GR'
    en_AG = 'en_AG'
    en_AU = 'en_AU'
    en_BE = 'en_BE'
    en_BW = 'en_BW'
    en_CA = 'en_CA'
    en_DK = 'en_DK'
    en_DL = 'en_DL'
    en_EN = 'en_EN'
    en_GB = 'en_GB'
    en_HK = 'en_HK'
    en_IE = 'en_IE'
    en_IL = 'en_IL'
    en_IN = 'en_IN'
    en_NG = 'en_NG'
    en_NZ = 'en_NZ'
    en_PH = 'en_PH'
    en_SC = 'en_SC'
    en_SG = 'en_SG'
    en_US = 'en_US'
    en_ZA = 'en_ZA'
    en_ZM = 'en_ZM'
    en_ZS = 'en_ZS'
    en_ZW = 'en_ZW'
    eo = 'eo'
    eo_EO = 'eo_EO'
    eo_US = 'eo_US'
    eo_XX = 'eo_XX'
    es_AR = 'es_AR'
    es_BO = 'es_BO'
    es_CL = 'es_CL'
    es_CO = 'es_CO'
    es_CR = 'es_CR'
    es_CU = 'es_CU'
    es_DO = 'es_DO'
    es_EC = 'es_EC'
    es_ES = 'es_ES'
    es_GT = 'es_GT'
    es_HN = 'es_HN'
    es_MX = 'es_MX'
    es_NI = 'es_NI'
    es_PA = 'es_PA'
    es_PE = 'es_PE'
    es_PR = 'es_PR'
    es_PY = 'es_PY'
    es_SV = 'es_SV'
    es_US = 'es_US'
    es_UY = 'es_UY'
    es_VE = 'es_VE'
    et_EE = 'et_EE'
    eu_ES = 'eu_ES'
    eu_FR = 'eu_FR'
    fa_IR = 'fa_IR'
    ff_SN = 'ff_SN'
    fi_FI = 'fi_FI'
    fil_PH = 'fil_PH'
    fo_FO = 'fo_FO'
    fr_BE = 'fr_BE'
    fr_CA = 'fr_CA'
    fr_CH = 'fr_CH'
    fr_FR = 'fr_FR'
    fr_LU = 'fr_LU'
    fur_IT = 'fur_IT'
    fy_DE = 'fy_DE'
    fy_NL = 'fy_NL'
    ga_IE = 'ga_IE'
    gd_GB = 'gd_GB'
    gez_ER = 'gez_ER'
    gez_ET = 'gez_ET'
    gl_ES = 'gl_ES'
    gu_IN = 'gu_IN'
    gv_GB = 'gv_GB'
    ha_NG = 'ha_NG'
    hak_TW = 'hak_TW'
    he_IL = 'he_IL'
    hi_IN = 'hi_IN'
    hif_FJ = 'hif_FJ'
    hne_IN = 'hne_IN'
    hr_HR = 'hr_HR'
    hsb_DE = 'hsb_DE'
    ht_HT = 'ht_HT'
    hu_HU = 'hu_HU'
    hy_AM = 'hy_AM'
    ia = 'ia'
    ia_FR = 'ia_FR'
    id_ID = 'id_ID'
    ig_NG = 'ig_NG'
    ik_CA = 'ik_CA'
    is_IS = 'is_IS'
    it_CH = 'it_CH'
    it_IT = 'it_IT'
    iu_CA = 'iu_CA'
    iw_IL = 'iw_IL'
    ja_JP = 'ja_JP'
    ka_GE = 'ka_GE'
    kab_DZ = 'kab_DZ'
    kk_KZ = 'kk_KZ'
    kl_GL = 'kl_GL'
    km_KH = 'km_KH'
    kn_IN = 'kn_IN'
    ko_KR = 'ko_KR'
    kok_IN = 'kok_IN'
    ks_IN = 'ks_IN'
    ku_TR = 'ku_TR'
    kw_GB = 'kw_GB'
    ky_KG = 'ky_KG'
    lb_LU = 'lb_LU'
    lg_UG = 'lg_UG'
    li_BE = 'li_BE'
    li_NL = 'li_NL'
    lij_IT = 'lij_IT'
    ln_CD = 'ln_CD'
    lo_LA = 'lo_LA'
    lt_LT = 'lt_LT'
    lv_LV = 'lv_LV'
    lzh_TW = 'lzh_TW'
    mag_IN = 'mag_IN'
    mai_IN = 'mai_IN'
    mai_NP = 'mai_NP'
    mfe_MU = 'mfe_MU'
    mg_MG = 'mg_MG'
    mhr_RU = 'mhr_RU'
    mi_NZ = 'mi_NZ'
    miq_NI = 'miq_NI'
    mjw_IN = 'mjw_IN'
    mk_MK = 'mk_MK'
    ml_IN = 'ml_IN'
    mn_MN = 'mn_MN'
    mni_IN = 'mni_IN'
    mr_IN = 'mr_IN'
    ms_MY = 'ms_MY'
    mt_MT = 'mt_MT'
    my_MM = 'my_MM'
    nan_TW = 'nan_TW'
    nb_NO = 'nb_NO'
    nds_DE = 'nds_DE'
    nds_NL = 'nds_NL'
    ne_NP = 'ne_NP'
    nhn_MX = 'nhn_MX'
    niu_NU = 'niu_NU'
    niu_NZ = 'niu_NZ'
    nl_AW = 'nl_AW'
    nl_BE = 'nl_BE'
    nl_NL = 'nl_NL'
    nn_NO = 'nn_NO'
    no_NO = 'no_NO'
    nr_ZA = 'nr_ZA'
    nso_ZA = 'nso_ZA'
    ny_NO = 'ny_NO'
    oc_FR = 'oc_FR'
    om_ET = 'om_ET'
    om_KE = 'om_KE'
    or_IN = 'or_IN'
    os_RU = 'os_RU'
    pa_IN = 'pa_IN'
    pa_PK = 'pa_PK'
    pap_AN = 'pap_AN'
    pap_AW = 'pap_AW'
    pap_CW = 'pap_CW'
    pd_DE = 'pd_DE'
    pd_US = 'pd_US'
    ph_PH = 'ph_PH'
    pl_PL = 'pl_PL'
    pp_AN = 'pp_AN'
    ps_AF = 'ps_AF'
    pt_BR = 'pt_BR'
    pt_PT = 'pt_PT'
    quz_PE = 'quz_PE'
    raj_IN = 'raj_IN'
    ro_RO = 'ro_RO'
    ru_RU = 'ru_RU'
    ru_UA = 'ru_UA'
    rw_RW = 'rw_RW'
    sa_IN = 'sa_IN'
    sat_IN = 'sat_IN'
    sc_IT = 'sc_IT'
    sd_IN = 'sd_IN'
    sd_PK = 'sd_PK'
    se_NO = 'se_NO'
    sgs_LT = 'sgs_LT'
    sh_HR = 'sh_HR'
    shn_MM = 'shn_MM'
    shs_CA = 'shs_CA'
    si_LK = 'si_LK'
    sid_ET = 'sid_ET'
    sk_SK = 'sk_SK'
    sl_CS = 'sl_CS'
    sl_SI = 'sl_SI'
    sm_WS = 'sm_WS'
    so_DJ = 'so_DJ'
    so_ET = 'so_ET'
    so_KE = 'so_KE'
    so_SO = 'so_SO'
    sq_AL = 'sq_AL'
    sq_MK = 'sq_MK'
    sr_CS = 'sr_CS'
    sr_ME = 'sr_ME'
    sr_RS = 'sr_RS'
    ss_ZA = 'ss_ZA'
    st_ZA = 'st_ZA'
    sv_FI = 'sv_FI'
    sv_SE = 'sv_SE'
    sw_KE = 'sw_KE'
    sw_TZ = 'sw_TZ'
    szl_PL = 'szl_PL'
    ta_IN = 'ta_IN'
    ta_LK = 'ta_LK'
    tcy_IN = 'tcy_IN'
    te_IN = 'te_IN'
    tg_TJ = 'tg_TJ'
    th_TH = 'th_TH'
    the_NP = 'the_NP'
    ti_ER = 'ti_ER'
    ti_ET = 'ti_ET'
    tig_ER = 'tig_ER'
    tk_TM = 'tk_TM'
    tl_PH = 'tl_PH'
    tn_ZA = 'tn_ZA'
    to_TO = 'to_TO'
    tpi_PG = 'tpi_PG'
    tr_CY = 'tr_CY'
    tr_TR = 'tr_TR'
    ts_ZA = 'ts_ZA'
    tt_RU = 'tt_RU'
    ug_CN = 'ug_CN'
    uk_UA = 'uk_UA'
    unm_US = 'unm_US'
    ur_IN = 'ur_IN'
    ur_PK = 'ur_PK'
    uz_UZ = 'uz_UZ'
    ve_ZA = 've_ZA'
    vi_VN = 'vi_VN'
    wa_BE = 'wa_BE'
    wae_CH = 'wae_CH'
    wal_ET = 'wal_ET'
    wo_SN = 'wo_SN'
    xh_ZA = 'xh_ZA'
    yi_US = 'yi_US'
    yo_NG = 'yo_NG'
    yue_HK = 'yue_HK'
    yuw_PG = 'yuw_PG'
    zh_CN = 'zh_CN'
    zh_HK = 'zh_HK'
    zh_SG = 'zh_SG'
    zh_TW = 'zh_TW'
    zu_ZA = 'zu_ZA'


class Determiner(models.TextChoices):
    a = 'a'
    an = 'an'
    the = 'the'
    blank = ''
    auto = 'auto'


class MimeType(models.TextChoices):
    application_javascript = 'application/javascript'
    application_json = 'application/json'
    application_manifest_json = 'application/manifest+json'
    application_msword = 'application/msword'
    application_octet_stream = 'application/octet-stream'
    application_oda = 'application/oda'
    application_pdf = 'application/pdf'
    application_pkcs7_mime = 'application/pkcs7-mime'
    application_postscript = 'application/postscript'
    application_rtf = 'application/rtf'
    application_vnd_apple_mpegurl = 'application/vnd.apple.mpegurl'
    application_vnd_ms_excel = 'application/vnd.ms-excel'
    application_vnd_ms_powerpoint = 'application/vnd.ms-powerpoint'
    application_wasm = 'application/wasm'
    application_x_bcpio = 'application/x-bcpio'
    application_x_cpio = 'application/x-cpio'
    application_x_csh = 'application/x-csh'
    application_x_dvi = 'application/x-dvi'
    application_x_gtar = 'application/x-gtar'
    application_x_hdf = 'application/x-hdf'
    application_x_hdf5 = 'application/x-hdf5'
    application_x_latex = 'application/x-latex'
    application_x_mif = 'application/x-mif'
    application_x_netcdf = 'application/x-netcdf'
    application_x_pkcs12 = 'application/x-pkcs12'
    application_x_pn_realaudio = 'application/x-pn-realaudio'
    application_x_python_code = 'application/x-python-code'
    application_x_sh = 'application/x-sh'
    application_x_shar = 'application/x-shar'
    application_x_shockwave_flash = 'application/x-shockwave-flash'
    application_x_sv4cpio = 'application/x-sv4cpio'
    application_x_sv4crc = 'application/x-sv4crc'
    application_x_tar = 'application/x-tar'
    application_x_tcl = 'application/x-tcl'
    application_x_tex = 'application/x-tex'
    application_x_texinfo = 'application/x-texinfo'
    application_x_troff = 'application/x-troff'
    application_x_troff_man = 'application/x-troff-man'
    application_x_troff_me = 'application/x-troff-me'
    application_x_troff_ms = 'application/x-troff-ms'
    application_x_ustar = 'application/x-ustar'
    application_x_wais_source = 'application/x-wais-source'
    application_xml = 'application/xml'
    application_zip = 'application/zip'
    audio_basic = 'audio/basic'
    audio_midi = 'audio/midi'
    audio_mpeg = 'audio/mpeg'
    audio_x_aiff = 'audio/x-aiff'
    audio_x_pn_realaudio = 'audio/x-pn-realaudio'
    audio_x_wav = 'audio/x-wav'
    image_gif = 'image/gif'
    image_ief = 'image/ief'
    image_jpeg = 'image/jpeg'
    image_jpg = 'image/jpg'
    image_pict = 'image/pict'
    image_png = 'image/png'
    image_svg_xml = 'image/svg+xml'
    image_tiff = 'image/tiff'
    image_vnd_microsoft_icon = 'image/vnd.microsoft.icon'
    image_x_cmu_raster = 'image/x-cmu-raster'
    image_x_ms_bmp = 'image/x-ms-bmp'
    image_x_portable_anymap = 'image/x-portable-anymap'
    image_x_portable_bitmap = 'image/x-portable-bitmap'
    image_x_portable_graymap = 'image/x-portable-graymap'
    image_x_portable_pixmap = 'image/x-portable-pixmap'
    image_x_rgb = 'image/x-rgb'
    image_x_xbitmap = 'image/x-xbitmap'
    image_x_xpixmap = 'image/x-xpixmap'
    image_x_xwindowdump = 'image/x-xwindowdump'
    message_rfc822 = 'message/rfc822'
    text_css = 'text/css'
    text_csv = 'text/csv'
    text_html = 'text/html'
    text_plain = 'text/plain'
    text_richtext = 'text/richtext'
    text_tab_separated_values = 'text/tab-separated-values'
    text_x_python = 'text/x-python'
    text_x_setext = 'text/x-setext'
    text_x_sgml = 'text/x-sgml'
    text_x_vcard = 'text/x-vcard'
    text_xml = 'text/xml'
    text_xul = 'text/xul'
    video_mp4 = 'video/mp4'
    video_mpeg = 'video/mpeg'
    video_quicktime = 'video/quicktime'
    video_webm = 'video/webm'
    video_x_msvideo = 'video/x-msvideo'
    video_x_sgi_movie = 'video/x-sgi-movie'
